@@ -108,10 +108,52 @@ def write_region(fname, ra, dec, src_flag, ra_back=-1, dec_back=-1, radius=120):
 	ff.close()
 
 
-def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, spec_flag=False, lc_flag=False,
-				 ds9_flag=False, ra_back=-1, dec_back=-1, radius=120, write_baryevtfile_flag=False,
+def get_src_from_obsid(obsid, repository_path=os.environ['HOME'] + '/NUSTAR/Repository'):
+	from astropy.io import fits as pf
+	search_str = repository_path + '/' + obsid + '/event_cl/nu*A*_cl.evt.gz'
+	logger.debug(search_str)
+	eventsA = glob(search_str)
+	if len(eventsA) == 0:
+		logger.warning('no event data for ' + obsid + '? Reurning none')
+		return None
+	with pf.open(eventsA[0]) as ff:
+		src = ff[1].header['OBJECT']
+	logger.debug('Found source ' + src)
+	return src
+
+def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, region_flag=False,
+				 spec_flag=False, lc_flag=False,
+				 no_ds9_flag=False, ra_back=-1, dec_back=-1, radius=120, write_baryevtfile_flag=False,
 				 filter_evt_flag=False, user_gti_file='none',
-				 repository_location=os.environ['HOME'] + '/NUSTAR/Repository', t_bin=100., e_min=3., e_max=30.):
+				 repository_location=os.environ['HOME'] + '/NUSTAR/Repository', t_bin=100., e_min=3., e_max=30.,
+				 J_A=None, J_B=None):
+	'''
+
+	:param obsid: the observation ID
+	:param ra_src: RA of the source
+	:param dec_src: Dec of the source
+	:param outdir_base: the basename for products
+	:param pipeline_flag: it tuns the pipeline
+	:param region_flag: it writes region (if not existent) and allows the user to check them
+	:param spec_flag: it extracts spectra
+	:param lc_flag: extracts light crves (set also t_bin e_min and e_max
+	:param no_ds9_flag: this is relevant just if region_flag is true, if set to True, it will use JS9, with the diplays
+						for FPMA and FPMB provided by the caller
+	:param ra_back: optional RA of the center of background region (otherise offset from the source)
+	:param dec_back: optional Dec of the center of background region (otherise offset from the source)
+	:param radius: the radius for source regions )defaults to 120
+	:param write_baryevtfile_flag: to be used with lightcurve to extract barycentric corrected event files
+	:param filter_evt_flag: This is to be used when lightcurves are extracted to extract barycentric corrected
+							event files for the source and backgroud for FPMA and FPMB
+	:param user_gti_file: if this is provided
+	:param repository_location: the location of the repository of raw data
+	:param t_bin: time bin for light curves
+	:param e_min: Minimum energy for lightcurve
+	:param e_max: Maximium energy for lightcurve
+	:param J_A:  handler for the JS9 display for FPMA
+	:param J_B:  handler for the JS9 display for FPMA
+	:return: zero
+	'''
 	if (not os.path.isdir(repository_location + "/" + obsid)):
 		logger.error("The obsid you specified (" + obsid + ") does not exist in the repository " + repository_location)
 		return 1
@@ -133,9 +175,96 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, spec_
 
 		cmd = "nupipeline indir=" + repository_location + "/" + obsid + " steminputs=nu" + obsid + " obsmode=Science outdir=" + outdir_pipeline
 
-		run(cmd)
+		return run(cmd)
 		# logfile.close()
 
+	if region_flag:
+		if not os.path.isdir(outdir_pipeline):
+			logger.error("You need to have processed the pipeline before spectral extraction. Folder %s does not exist" % (
+				outdir_pipeline))
+			return 1
+		outdir = outdir_base + "_spec"
+		outdir_pipeline = outdir_base + "_pipeline"
+		prepare_outdir(outdir)
+
+		logger.info("We store regions in the spectral extraction folder '%s'" % outdir)
+
+		src_regionA = outdir + "/sourceA.reg"
+		if (not os.path.isfile(src_regionA)):
+			write_region(src_regionA, ra_src, dec_src, True)
+		src_regionB = outdir + "/sourceB.reg"
+		if (not os.path.isfile(src_regionB)):
+			write_region(src_regionB, ra_src, dec_src, True)
+		bkg_regionA = outdir + "/backgroundA.reg"
+		if (not os.path.isfile(bkg_regionA)):
+			write_region(bkg_regionA, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
+		bkg_regionB = outdir + "/backgroundB.reg"
+		if (not os.path.isfile(bkg_regionB)):
+			write_region(bkg_regionB, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
+
+
+		if no_ds9_flag==False:
+			logger.info("Please check the regions for FPMA")
+			cmd = "ds9 -scale log " + outdir_pipeline + "/nu" + obsid + "A01_cl.evt -regions " + src_regionA + " -regions " + bkg_regionA
+			run(cmd)
+			logger.info("Please check the regions for FPMB")
+			cmd = "ds9 -scale log " + outdir_pipeline + "/nu" + obsid + "B01_cl.evt -regions " + src_regionB + " -regions " + bkg_regionB
+			run(cmd)
+
+			#logger.info("Have you checked properly the region files?(y/Y will continue the processing any other key stop it)")
+			logger.warning("We ask to rerun the analysis disabling the ds9 iteractive command with which you have checked the region files")
+			# ans = input("Please enter the key: ...")
+			# if (not (ans == 'y')) and (not (ans == 'Y')):
+			# 	print("Exit processing")
+			return 0
+		else:
+			# JS9 does not load event files, I need to create an image first
+
+			xsel_cmd = '''dummy
+			read eve
+			./ 
+			nu%s%s01_cl.evt 
+			yes 
+			extra ima
+			save ima ima%s.fits
+
+
+
+
+			quit
+
+
+
+
+
+			'''
+			os.chdir(outdir_pipeline)
+			for unit in ['A', 'B']:
+				with open('xsel_%s.txt' % unit, 'w') as ff:
+					ff.write(xsel_cmd % (obsid, unit, unit))
+				cmd = 'xselect < ' + 'xsel_%s.txt' % (unit)
+				run(cmd)
+			os.chdir('..')
+
+			logger.warning("Please check the regions for FPMA")
+
+			if J_A is None or J_B is None:
+				raise RuntimeError('We need a JS9 display for FPMA and FPMB to check regions and events, leave ds9_flag=False to use ds9')
+			J_A.Load(outdir_pipeline + "/imaA.fits")
+			J_A.LoadRegions(src_regionA)
+			J_A.LoadRegions(bkg_regionA)
+			J_A.send({'cmd': 'SetScale', 'args': ['log']})
+			J_A.SetColormap('red')
+
+			logger.warning("Please check the regions for FPMB")
+
+			J_B.Load(outdir_pipeline + "/imaB.fits")
+			J_B.LoadRegions(src_regionB)
+			J_B.LoadRegions(bkg_regionB)
+			J_B.send({'cmd': 'SetScale', 'args': ['log']})
+			J_B.SetColormap('red')
+			return 0
+
 	if lc_flag:
 
 		import shutil
@@ -157,17 +286,22 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, spec_
 		logger.info("Energy scale E = Channel Number * 0.04 keV + 1.6 keV\n")
 		logger.info("E_min=%f E_max=%f t_bin=%f ch_min=%d ch_max=%d\n" % (e_min, e_max, t_bin, ch_min, ch_max))
 
+
+		if ((not os.path.isfile(outdir_base + "_spec/sourceA.reg")) or \
+				(not os.path.isfile(outdir_base + "_spec/sourceB.reg")) or \
+				(not os.path.isfile(outdir_base + "_spec/backgroundA.reg")) or \
+				(not os.path.isfile(outdir_base + "_spec/backgroundB.reg")) ):
+			raise RuntimeError('You need to have region files in %s_spec make them with the option region_flag' % outdir_base)
+
 		src_region = outdir + "/sourceA.reg"
 		if (not os.path.isfile(src_region)):
-			write_region(src_region, ra_src, dec_src, True)
+			spec_src = outdir_base + "_spec/sourceA.reg"
+			shutil.copy(spec_src, src_region)
 
 		bkg_region = outdir + "/backgroundA.reg"
 		if (not os.path.isfile(bkg_region)):
 			spec_bkg = outdir_base + "_spec/backgroundA.reg"
-			if (os.path.isfile(spec_bkg)):
-				shutil.copy(spec_bkg, bkg_region)
-			else:
-				write_region(bkg_region, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
+			shutil.copy(spec_bkg, bkg_region)
 
 		cmd = "nuproducts indir=" + outdir_pipeline + " instrument=FPMA steminputs=nu" + obsid + " stemout=FPMA_%.1f_%.1f outdir=" % (
 		e_min, e_max) + outdir
@@ -191,15 +325,13 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, spec_
 
 		src_region = outdir + "/sourceB.reg"
 		if (not os.path.isfile(src_region)):
-			write_region(src_region, ra_src, dec_src, True)
+			spec_src = outdir_base + "_spec/sourceB.reg"
+			shutil.copy(spec_src, src_region)
 
 		bkg_region = outdir + "/backgroundB.reg"
 		if (not os.path.isfile(bkg_region)):
 			spec_bkg = outdir_base + "_spec/backgroundB.reg"
-			if (os.path.isfile(spec_bkg)):
-				shutil.copy(spec_bkg, bkg_region)
-			else:
-				write_region(bkg_region, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
+			shutil.copy(spec_bkg, bkg_region)
 
 		cmd = "nuproducts indir=" + outdir_pipeline + " instrument=FPMB steminputs=nu" + obsid + " stemout=FPMB_%.1f_%.1f outdir=" % (
 		e_min, e_max) + outdir
@@ -244,7 +376,7 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, spec_
 			for unit in ['A', 'B']:
 				for reg in ['source', 'background']:
 					with open('xsel_%s_%s.txt' % (reg, unit), 'w') as ff:
-						ff.write(xsel_cmd % ('A', e_min, e_max, reg, unit, reg, unit))
+						ff.write(xsel_cmd % (unit, e_min, e_max, reg, unit, reg, unit))
 					cmd = 'xselect < ' + 'xsel_%s_%s.txt' % (reg, unit)
 					run(cmd)
 			os.chdir('..')
@@ -260,33 +392,15 @@ def wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag=False, spec_
 		prepare_outdir(outdir)
 
 		src_regionA = outdir + "/sourceA.reg"
-		if (not os.path.isfile(src_regionA)):
-			write_region(src_regionA, ra_src, dec_src, True)
 		src_regionB = outdir + "/sourceB.reg"
-		if (not os.path.isfile(src_regionB)):
-			write_region(src_regionB, ra_src, dec_src, True)
 		bkg_regionA = outdir + "/backgroundA.reg"
-		if (not os.path.isfile(bkg_regionA)):
-			write_region(bkg_regionA, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
 		bkg_regionB = outdir + "/backgroundB.reg"
-		if (not os.path.isfile(bkg_regionB)):
-			write_region(bkg_regionB, ra_src, dec_src, False, ra_back=ra_back, dec_back=dec_back, radius=radius)
 
-
-		if not ds9_flag:
-			logger.info("Please check the regions for FPMA")
-			cmd = "ds9 -scale log " + outdir_pipeline + "/nu" + obsid + "A01_cl.evt -regions " + src_regionA + " -regions " + bkg_regionA
-			run(cmd)
-			logger.info("Please check the regions for FPMB")
-			cmd = "ds9 -scale log " + outdir_pipeline + "/nu" + obsid + "B01_cl.evt -regions " + src_regionB + " -regions " + bkg_regionB
-			run(cmd)
-
-			#logger.info("Have you checked properly the region files?(y/Y will continue the processing any other key stop it)")
-			logger.warning("We ask to rerun the analysis disabling the ds9 iteractive command with which you have checked the region files")
-			# ans = input("Please enter the key: ...")
-			# if (not (ans == 'y')) and (not (ans == 'Y')):
-			# 	print("Exit processing")
-			return 0
+		if ((not os.path.isfile(src_regionA)) or \
+				(not os.path.isfile(src_regionB)) or \
+				(not os.path.isfile(bkg_regionA)) or \
+				(not os.path.isfile(bkg_regionB))):
+			raise RuntimeError('You need to have region files in %s make them with the option region_flag' % outdir)
 
 		cmd = "cd " + outdir + ";nuproducts indir=../" + outdir_pipeline + " instrument=FPMA steminputs=nu" + obsid + " stemout=FPMA outdir=."  # +outdir
 		cmd += " srcregionfile=../" + src_regionA + " bkgregionfile=../" + bkg_regionA + " bkgextract=yes runbackscale=yes cleanup=yes "
@@ -331,7 +445,8 @@ def process():
 	parser.add_argument("Dec", metavar='dec', nargs=1, help="Dec of the source", type=float)
 	parser.add_argument('outdir', metavar='outdir', type=str, nargs=1,
 						help='output folder base name (to be appended with _pipeline, _lc, or _spec')
-
+	parser.add_argument("--regions", help="Writes (if not present) and checks regions with ds9 (no js9)",
+						action='store_true')
 	parser.add_argument("--pipeline", help="Run pipeline", action='store_true')
 	parser.add_argument("--spec", help="Run spectral extraction", action='store_true')
 	parser.add_argument("--lightcurve", help="Run light curve extraction", action='store_true')
@@ -383,6 +498,7 @@ def process():
 	ra_src = args.RA[0]
 	dec_src = args.Dec[0]
 	pipeline_flag = args.pipeline
+	region_flag = args.regions
 	lc_flag = args.lightcurve
 	spec_flag = args.spec
 	ds9_flag = args.no_ds9
@@ -396,7 +512,7 @@ def process():
 	e_max = args.eMax
 	user_gti_file = args.usergti
 
-	out = wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag, spec_flag, lc_flag,
+	out = wrap_process(obsid, ra_src, dec_src, outdir_base, pipeline_flag, region_flag, spec_flag, lc_flag,
 				 ds9_flag, ra_back, dec_back, radius, write_baryevtfile_flag, filter_evt_flag,
 				 user_gti_file, repository_location, t_bin, e_min, e_max)
 
